@@ -5,13 +5,11 @@ Module providing easy API for working with remote files and folders.
 from __future__ import with_statement
 
 import hashlib
-import tempfile
-import re
 import os
 from StringIO import StringIO
 from functools import partial
 
-from fabric.api import *
+from fabric.api import run, sudo, hide, settings, env, put, abort
 from fabric.utils import apply_lcwd
 
 
@@ -25,9 +23,13 @@ def exists(path, use_sudo=False, verbose=False):
     stderr and any warning resulting from the file not existing) in order to
     avoid cluttering output. You may specify ``verbose=True`` to change this
     behavior.
+
+    .. versionchanged:: 1.13
+        Replaced internal use of ``test -e`` with ``stat`` for improved remote
+        cross-platform (e.g. Windows) compatibility.
     """
     func = use_sudo and sudo or run
-    cmd = 'test -e %s' % _expand_path(path)
+    cmd = 'stat %s' % _expand_path(path)
     # If verbose, run normally
     if verbose:
         with settings(warn_only=True):
@@ -224,7 +226,7 @@ def sed(filename, before, after, limit='', use_sudo=False, backup='.bak',
     # Test the OS because of differences between sed versions
 
     with hide('running', 'stdout'):
-        platform = run("uname")
+        platform = run("uname", shell=False, pty=False)
     if platform in ('NetBSD', 'OpenBSD', 'QNX'):
         # Attempt to protect against failures/collisions
         hasher = hashlib.sha1()
@@ -348,7 +350,7 @@ def contains(filename, text, exact=False, use_sudo=False, escape=True,
     added.)
 
     The ``shell`` argument will be eventually passed to ``run/sudo``. See
-    description of the same argumnet in ``~fabric.contrib.sed`` for details.
+    description of the same argument in ``~fabric.contrib.sed`` for details.
 
     If ``case_sensitive`` is False, the `-i` flag will be passed to ``egrep``.
 
@@ -430,14 +432,42 @@ def append(filename, text, use_sudo=False, partial=False, escape=True,
 
 def _escape_for_regex(text):
     """Escape ``text`` to allow literal matching using egrep"""
-    regex = re.escape(text)
-    # Seems like double escaping is needed for \
-    regex = regex.replace('\\\\', '\\\\\\')
-    # Triple-escaping seems to be required for $ signs
-    regex = regex.replace(r'\$', r'\\\$')
-    # Whereas single quotes should not be escaped
-    regex = regex.replace(r"\'", "'")
-    return regex
+    re_specials = '\\^$|(){}[]*+?.'
+    sh_specials = '\\$`"'
+    re_chars = []
+    sh_chars = []
+
+    for c in text:
+        if c in re_specials:
+            re_chars.append('\\')
+        re_chars.append(c)
+
+    for c in re_chars:
+        if c in sh_specials:
+            sh_chars.append('\\')
+        sh_chars.append(c)
+
+    return ''.join(sh_chars)
+
+def is_win():
+    """
+    Return True if remote SSH server is running Windows, False otherwise.
+
+    The idea is based on echoing quoted text: \*NIX systems will echo quoted
+    text only, while Windows echoes quotation marks as well.
+    """
+    with settings(hide('everything'), warn_only=True):
+        return '"' in run('echo "Will you echo quotation marks"')
 
 def _expand_path(path):
-    return '"$(echo %s)"' % path
+    """
+    Return a path expansion
+
+    E.g.    ~/some/path     ->  /home/myuser/some/path
+            /user/\*/share   ->  /user/local/share
+    More examples can be found here: http://linuxcommand.org/lc3_lts0080.php
+
+    .. versionchanged:: 1.0
+        Avoid breaking remote Windows commands which does not support expansion.
+    """
+    return path if is_win() else '"$(echo %s)"' % path
